/* Shortcut Launcher v1.0
 *
 * Copyright (C) 2006 BoxiT Software.
 *
 * ShortcutLauncher.js
 *
 * Requires Yahoo! Widget Engine 3 or later
 */
 
 	// Include the other scripts to handle UI and Shortcuts
 	include("Resources/js/UserInterface.js");
 	include("Resources/js/ShortcutManagement.js");
 	
 	// Set the path to the widgets data folder and add the shortcuts.xml file
	var xmlPath = system.widgetDataFolder + "\\Shortcuts.xml";
	
	// Check if this is the first run of the widget
	checkFirstRun(xmlPath);
	
	// Set a global variable to hold the shortcut xml data
 	var xmlData = loadShortcutXmlData(xmlPath);
 	
 	// Global variable to the root xml node
 	var xmlRootNode = "";
 	
	updatePageDisplay();
					
	function loadShortcutXmlData(xmlPath)
	{
		// Check that the shortcut xml file exists
		if (filesystem.itemExists(xmlPath))
		{
			var data = filesystem.readFile(xmlPath);
			var xml = XMLDOM.parse(data);
			var results = xml.evaluate("Shortcuts");
			xmlRootNode = results.item(0);
		} else{
			// The xml file does not exist so we create the base structure in the xmlData variable
			var xml = XMLDOM.createDocument();
			xmlRootNode = xml.createElement("Shortcuts");
			xml.appendChild(xmlRootNode);
		}
		
		// Return the resultant xml data
		return xml;	
	}
		
	function processDroppedFiles(fileName)
	{
		// Get an array of the categories in existance
		var categories = getCategories();
			
		// We need to handle the possibility that categories don't exist, especially if the widget is running for the first time
		if (categories == null)
		{
			manageCategories('Add');
			var strCategory = preferences.currentCategory.value;
		} else{
			// Present a list of available categories for the user to choose
			var category = categorySelector("Select category to add the dropped shortcuts to", "Add Shortcuts - Select Category", "Select");
			if (category != null)
			{
				var strCategory = category;
			} else{
				alert("No category selected.  Shortcuts will not be created for the dropped files.");
				return;
			}
		}
		
		for (i = 1; i < fileName.length; i++)
		{
			// We need to do things differently depending on the type of file dropped
			if (fileName[i].substring(fileName[i].length - 3) == "url")
			{
				var lines = filesystem.readFile(fileName[i], true);
				var strPath = getURL(lines);
				var strTitle = filesystem.getDisplayName(fileName[i]).substring(-4);
			}else{
				// Get the properties for the shortcut
				var strPath = fileName[i];
				var strTitle = filesystem.getDisplayName(strPath);
			}			
			
			// Add the shortcut data to the xml
			var shortcutNode = xmlData.createElement("Shortcut");
			shortcutNode.setAttribute("Path", strPath);
			shortcutNode.setAttribute("Title", strTitle);
			shortcutNode.setAttribute("Category", strCategory);
			var results = xmlData.evaluate("Shortcuts");
			results.item(0).appendChild(shortcutNode);
		}
		updatePageDisplay();
	}
	
	function getURL(lines)
	{
		for (i = 0; i < lines.length; i++)
		{
			print(lines[i]);
			if (lines[i].substring(0, 4) == "URL=")
			{
				return lines[i].substring(4);
			}
		}	
	}
	
	function processDroppedWebUrls(urls)
	{
		// Get an array of the categories in existance
		var categories = getCategories();
			
		// We need to handle the possibility that categories don't exist, especially if the widget is running for the first time
		if (categories == null)
		{
			manageCategories('Add');
			var strCategory = preferences.currentCategory.value;
		} else{
			// Present a list of available categories for the user to choose
			var category = categorySelector("Select category to add the dropped shortcuts to", "Add Shortcuts - Select Category", "Select");
			if (category != null)
			{
				var strCategory = category;
			} else{
				alert("No category selected.  Shortcuts will not be created for the dropped files.");
				return;
			}
		}
		
		for (i = 1; i < urls.length; i++)
		{
			// Get the properties for the shortcut
			var strPath = urls[i];
			var strTitle = urls[i];
			
			// Add the shortcut data to the xml
			var shortcutNode = xmlData.createElement("Shortcut");
			shortcutNode.setAttribute("Path", strPath);
			shortcutNode.setAttribute("Title", strTitle);
			shortcutNode.setAttribute("Category", strCategory);
			var results = xmlData.evaluate("Shortcuts");
			results.item(0).appendChild(shortcutNode);
		}
		updatePageDisplay();
	}
	
	function shortcutMouseDown(shortcutID)
	{
		var shortcutItems = getShortcuts(preferences.currentCategory.value);
		var strPath = shortcutItems[shortcutID].path;
		if (strPath.substr(0, 4) == "http")
		{
			openURL(strPath);	
		} else{
			// The method of opening depends on the extension..
			if (strPath.substring(strPath.length - 3) == "lnk")
			{
				var strCommand = "start -D \"" + convertPathToPlatform(strPath) + "\"";
				runCommand(strCommand);
			} else{
				filesystem.open(strPath);
			}
		}
	}
	
	function checkFirstRun(xmlPath)
	{
		// We need to check for the existence of the xml file
		if (filesystem.itemExists(xmlPath) == false)
		{
			// Create the popup option lists
			var viewTypes = ["Text", "Icons", "Icon & Text"];
			var iconSizes = ["16", "24", "32"];
			
			// The xml file does not exist so we display an initial settings form
			var formFields = Array();
			
			formFields[0] = new FormField();
			formFields[0].name = "viewType";
			formFields[0].type = "popup";
			formFields[0].title = "View Type:";
			formFields[0].option = viewTypes;
			formFields[0].defaultValue = "Icons";
			formFields[0].description = "The type of view layout to use for the initial display.";
			
			formFields[1] = new FormField();
			formFields[1].name = "iconSize";
			formFields[1].type = "popup";
			formFields[1].title = "Icon Size:";
			formFields[1].option = iconSizes;
			formFields[1].defaultValue = "24";
			formFields[1].description = "The size of icons when the icon view type is selected.";
			
			formFields[2] = new FormField();
			formFields[2].name = "highlightColour";
			formFields[2].type = "color";
			formFields[2].title = "Highlight Colour:";
			formFields[2].defaultValue = "#FF0000";
			formFields[2].description = "Select what colour the highlighter should be when the mouse is over a shortcut.";
			
			formResults = form(formFields, "Set Initial Display Settings", "Continue", "Cancel");
			
			if (formResults != null)
			{
				preferences.viewType.value = formResults[0];
				preferences.iconSize.value = formResults[1];
				preferences.highlightColour.value = formResults[2];
			} else{
				alert("Default preferences will be set!");
				preferences.viewType.value = "Text";
				preferences.iconSize.value = "24";
				preferences.highlightColour.value = "#FF0000";
			}
		}
	}